package net.sergeych.kiloparsec

import kotlinx.coroutines.CancellationException
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import net.sergeych.crypto2.SigningKey
import net.sergeych.kiloparsec.adapter.InetTransportDevice
import net.sergeych.mp_logger.LogTag
import net.sergeych.mp_logger.debug
import net.sergeych.mp_logger.exception
import net.sergeych.mp_logger.info
import net.sergeych.mp_tools.globalLaunch
import net.sergeych.tools.AtomicCounter

private val instances = AtomicCounter()
@Suppress("unused")
class KiloServer<S>(
    private val clientInterface: KiloInterface<S>,
    private val connections: Flow<InetTransportDevice>,
    private val serverSecretKey: SigningKey? = null,
    private val sessionBuilder: ()->S,
    ): LogTag("KS:${instances.incrementAndGet()}") {

    private val job = globalLaunch {
        connections.collect { device ->
            launch {
                try {
                    info { "connected ${device}" }
                    KiloServerConnection(clientInterface, device, sessionBuilder(), serverSecretKey)
                        .apply { debug { "server connection is ready" }}
                        .run()
                }
                catch(_: CancellationException) {
                }
                catch(cce: LocalInterface.BreakConnectionException) {
                    info { "Closed exception caught, closing (${cce.flushSendQueue}" }
                }
                catch (t: Throwable) {
                    exception { "unexpected while creating kiloclient" to t }
                }
            }
        }
    }

    fun close() {
        println("PRREEEC")
        job.cancel()
        println("POOOSTC")
    }
}